@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -18,8 +17,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -36,6 +33,10 @@ from tornado.websocket import WebSocketError
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'WebSocketClientConnectionWrapper',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -44,9 +45,8 @@ from tornado.websocket import WebSocketError
 # Dev API
 #-----------------------------------------------------------------------------
 
-@dev((1,0,0))
 class WebSocketClientConnectionWrapper(object):
-    ''' Used for compat across Tornado versions and to add write_lock'''
+    ''' Used for compatibility across Tornado versions and to add write_lock'''
 
     def __init__(self, socket):
         if socket is None:
@@ -59,7 +59,6 @@ class WebSocketClientConnectionWrapper(object):
     # Internal methods --------------------------------------------------------
 
     @gen.coroutine
-    @dev((1,0,0))
     def write_message(self, message, binary=False, locked=True):
         ''' Write a message to the websocket after obtaining the appropriate
         Bokeh Document lock.
@@ -86,12 +85,10 @@ class WebSocketClientConnectionWrapper(object):
         else:
             write_message_unlocked()
 
-    @dev((1,0,0))
     def close(self, code=None, reason=None):
         ''' Close the websocket. '''
         return self._socket.close(code, reason)
 
-    @dev((1,0,0))
     def read_message(self, callback=None):
         ''' Read a message from websocket and execute a callback.
 
