@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 '''
 
 To run a Bokeh application on a Bokeh server from a single Python script,
@@ -115,14 +121,16 @@ Bokeh server instances behind a load balancer.
 
 By default, cross site connections to the Bokeh server websocket are not
 allowed. You can enable websocket connections originating from additional
-hosts by specifying them with the ``--allow-websocket-origin`` option:
+hosts by specifying them with the ``BOKEH_ALLOW_WS_ORIGIN`` environment variable
+or the ``--allow-websocket-origin`` option:
 
 .. code-block:: sh
 
     bokeh serve app_script.py --allow-websocket-origin foo.com:8081
 
 It is possible to specify multiple allowed websocket origins by adding
-the ``--allow-websocket-origin`` option multiple times.
+the ``--allow-websocket-origin`` option multiple times and to provide a
+comma separated list of hosts to ``BOKEH_ALLOW_WS_ORIGIN``
 
 The Bokeh server can also add an optional prefix to all URL paths.
 This can often be useful in conjunction with "reverse proxy" setups.
@@ -295,13 +303,28 @@ The log format can be controlled by the ``--log-format`` argument:
 The default log format is ``"{DEFAULT_LOG_FORMAT}"``
 
 '''
-from __future__ import absolute_import
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
 log = logging.getLogger(__name__)
 
-import argparse
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
 
+# Standard library imports
+import argparse
+from fnmatch import fnmatch
+import os
+
+# External imports
+from tornado.autoreload import watch
+
+# Bokeh imports
 from bokeh.application import Application
 from bokeh.resources import DEFAULT_SERVER_PORT
 from bokeh.util.logconfig import basicConfig
@@ -309,8 +332,6 @@ from bokeh.util.string import nice_join, format_docstring
 from bokeh.server.tornado import DEFAULT_WEBSOCKET_MAX_MESSAGE_SIZE_BYTES
 from bokeh.settings import settings
 
-from os import getpid
-
 from ..subcommand import Subcommand
 from ..util import build_single_handler_applications, die, report_server_init_errors
 
@@ -318,12 +339,9 @@ LOGLEVELS = ('trace', 'debug', 'info', 'warning', 'error', 'critical')
 SESSION_ID_MODES = ('unsigned', 'signed', 'external-signed')
 DEFAULT_LOG_FORMAT = "%(asctime)s %(message)s"
 
-__doc__ = format_docstring(__doc__,
-    DEFAULT_PORT=DEFAULT_SERVER_PORT,
-    LOGLEVELS=nice_join(LOGLEVELS),
-    SESSION_ID_MODES=nice_join(SESSION_ID_MODES),
-    DEFAULT_LOG_FORMAT=DEFAULT_LOG_FORMAT
-)
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
 
 base_serve_args = (
     ('--port', dict(
@@ -363,6 +381,14 @@ base_serve_args = (
     )),
 )
 
+__all__ = (
+    'Serve',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 class Serve(Subcommand):
     ''' Subcommand to launch the Bokeh server.
 
@@ -482,6 +508,19 @@ class Serve(Subcommand):
             type=int,
         )),
 
+        ('--dev', dict(
+            metavar ='FILES-TO-WATCH',
+            action  ='store',
+            default = None,
+            type    = str,
+            nargs   = '*',
+            help    =   "Enable live reloading during app development."
+                        "By default it watches all *.py *.html *.css *.yaml files"
+                        "in the app directory tree. Additional files can be passed"
+                        "as arguments."
+                        "NOTE: This setting only works with a single app."
+                        "It also restricts the number of processes to 1.",
+        )),
     )
 
 
@@ -558,6 +597,39 @@ class Serve(Subcommand):
 
         server_kwargs['use_index'] = not args.disable_index
         server_kwargs['redirect_root'] = not args.disable_index_redirect
+        server_kwargs['autoreload'] = args.dev is not None
+
+        def find_autoreload_targets(app_path):
+            path = os.path.abspath(app_path)
+            if not os.path.isdir(path):
+                return
+
+            for path, subdirs, files in os.walk(path):
+                for name in files:
+                    if (fnmatch(name, '*.html') or
+                        fnmatch(name, '*.css') or
+                        fnmatch(name, '*.yaml')):
+                        log.info("Watching: " + os.path.join(path, name))
+                        watch(os.path.join(path, name))
+
+        def add_optional_autoreload_files(file_list):
+            for filen in file_list:
+                if os.path.isdir(filen):
+                    log.warning("Cannot watch directory " + filen)
+                    continue
+                log.info("Watching: " + filen)
+                watch(filen)
+
+        if server_kwargs['autoreload']:
+            if len(applications.keys()) != 1:
+                die("--dev can only support a single app.")
+            if server_kwargs['num_procs'] != 1:
+                log.info("Running in --dev mode. --num-procs is limited to 1.")
+                server_kwargs['num_procs'] = 1
+
+            find_autoreload_targets(args.files[0])
+            add_optional_autoreload_files(args.dev)
+
 
         with report_server_init_errors(**server_kwargs):
             server = Server(applications, **server_kwargs)
@@ -577,5 +649,24 @@ class Serve(Subcommand):
                 url = "http://%s:%d%s%s" % (address_string, server.port, server.prefix, route)
                 log.info("Bokeh app running at: %s" % url)
 
-            log.info("Starting Bokeh server with process id: %d" % getpid())
+            log.info("Starting Bokeh server with process id: %d" % os.getpid())
             server.run_until_shutdown()
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
+
+__doc__ = format_docstring(__doc__,
+    DEFAULT_PORT=DEFAULT_SERVER_PORT,
+    LOGLEVELS=nice_join(LOGLEVELS),
+    SESSION_ID_MODES=nice_join(SESSION_ID_MODES),
+    DEFAULT_LOG_FORMAT=DEFAULT_LOG_FORMAT
+)
