@@ -25,25 +25,27 @@
     var id = cell.output_area._bokeh_element_id;
     var server_id = cell.output_area._bokeh_server_id;
     // Clean up Bokeh references
-    if (id !== undefined) {
+    if (id != null && id in Bokeh.index) {
       Bokeh.index[id].model.document.clear();
       delete Bokeh.index[id];
     }
 
     if (server_id !== undefined) {
       // Clean up Bokeh references
-      var cmd = "from bokeh.io import _state; print(_state.uuid_to_server['" + server_id + "'].get_sessions()[0].document.roots[0]._id)";
+      var cmd = "from bokeh.io.state import curstate; print(curstate().uuid_to_server['" + server_id + "'].get_sessions()[0].document.roots[0]._id)";
       cell.notebook.kernel.execute(cmd, {
         iopub: {
           output: function(msg) {
-            var element_id = msg.content.text.trim();
-            Bokeh.index[element_id].model.document.clear();
-            delete Bokeh.index[element_id];
+            var id = msg.content.text.trim();
+            if (id in Bokeh.index) {
+              Bokeh.index[id].model.document.clear();
+              delete Bokeh.index[id];
+            }
           }
         }
       });
       // Destroy server and session
-      var cmd = "from bokeh import io; io._destroy_server('" + server_id + "')";
+      var cmd = "import bokeh.io.notebook as ion; ion.destroy_server('" + server_id + "')";
       cell.notebook.kernel.execute(cmd);
     }
   }
@@ -60,10 +62,10 @@
       return
     }
 
-    var toinsert = output_area.element.find(`.${CLASS_NAME.split(' ')[0]}`);
+    var toinsert = output_area.element.find("." + CLASS_NAME.split(' ')[0]);
 
     if (output.metadata[EXEC_MIME_TYPE]["id"] !== undefined) {
-      toinsert[0].firstChild.textContent = output.data[JS_MIME_TYPE];
+      toinsert[toinsert.length - 1].firstChild.textContent = output.data[JS_MIME_TYPE];
       // store reference to embed id on output_area
       output_area._bokeh_element_id = output.metadata[EXEC_MIME_TYPE]["id"];
     }
@@ -72,7 +74,7 @@
       bk_div.innerHTML = output.data[HTML_MIME_TYPE];
       var script_attrs = bk_div.children[0].attributes;
       for (var i = 0; i < script_attrs.length; i++) {
-        toinsert[0].firstChild.setAttribute(script_attrs[i].name, script_attrs[i].value);
+        toinsert[toinsert.length - 1].firstChild.setAttribute(script_attrs[i].name, script_attrs[i].value);
       }
       // store reference to server id on output_area
       output_area._bokeh_server_id = output.metadata[EXEC_MIME_TYPE]["server_id"];
@@ -91,7 +93,7 @@
       this.keyboard_manager.register_events(toinsert);
       // Render to node
       var props = {data: data, metadata: metadata[EXEC_MIME_TYPE]};
-      render(props, toinsert[0]);
+      render(props, toinsert[toinsert.length - 1]);
       element.append(toinsert);
       return toinsert
     }
