@@ -1,11 +1,18 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-'''
+''' Provide functions for embedding Bokeh standalone and server content in
+web pages.
+
+.. autofunction:: autoload_static
+.. autofunction:: components
+.. autofunction:: file_html
+.. autofunction:: json_item
+.. autofunction:: server_document
+.. autofunction:: server_session
 
 '''
 
@@ -17,8 +24,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -29,6 +34,19 @@ from bokeh.util.api import general, dev ; general, dev
 
 # Bokeh imports
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'autoload_static',
+    'components',
+    'file_html',
+    'json_item',
+    'server_document',
+    'server_session',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -39,11 +57,16 @@ from .server import server_session
 from .standalone import autoload_static
 from .standalone import components
 from .standalone import file_html
+from .standalone import json_item
 
-__all__ = (
-    'autoload_static',
-    'components',
-    'file_html',
-    'server_document',
-    'server_session',
-)
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
