@@ -1,26 +1,63 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide a base class for all objects (called Bokeh Models) that can go in
 a Bokeh |Document|.
 
 '''
-from __future__ import absolute_import, print_function
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 import logging
-logger = logging.getLogger(__file__)
+log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 from json import loads
 from operator import itemgetter
 
-from six import iteritems
+# External imports
+from six import iteritems, string_types
 
+# Bokeh imports
 from .core.json_encoder import serialize_json
 from .core.properties import Any, Dict, Instance, List, String
 from .core.has_props import HasProps, MetaHasProps
 from .core.query import find
+
+from .events import Event
 from .themes import default as default_theme
+
 from .util.callback_manager import PropertyCallbackManager, EventCallbackManager
 from .util.future import with_metaclass
 from .util.serialization import make_id
-from .events import Event
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'collect_models',
+    'get_class',
+    'Model',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 def collect_models(*input_values):
     ''' Collect a duplicate-free list of all other Bokeh models referred to by
@@ -44,7 +81,7 @@ def collect_models(*input_values):
     queued = []
 
     def queue_one(obj):
-        if obj._id not in ids:
+        if obj.id not in ids:
             queued.append(obj)
 
     for value in input_values:
@@ -52,8 +89,8 @@ def collect_models(*input_values):
 
     while queued:
         obj = queued.pop(0)
-        if obj._id not in ids:
-            ids.add(obj._id)
+        if obj.id not in ids:
+            ids.add(obj.id)
             collected.append(obj)
             _visit_immediate_value_references(obj, queue_one)
 
@@ -94,6 +131,10 @@ def get_class(view_model_name):
     else:
         raise KeyError("View model name '%s' not found" % view_model_name)
 
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 class MetaModel(MetaHasProps):
     ''' Specialize the construction of |Model| classes.
 
@@ -216,14 +257,18 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
     def __init__(self, **kwargs):
         self._id = kwargs.pop("id", make_id())
         self._document = None
+        self._temp_document = None
         super(Model, self).__init__(**kwargs)
         default_theme.apply_to_model(self)
 
     def __str__(self):
-        return "%s(id=%r, ...)" % (self.__class__.__name__, getattr(self, "_id", None))
+        return "%s(id=%r, ...)" % (self.__class__.__name__, getattr(self, "id", None))
 
     __repr__ = __str__
 
+    @property
+    def id(self):
+        return self._id
 
     name = String(help="""
     An arbitrary, user-supplied name for this model.
@@ -259,7 +304,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         [GlyphRenderer(id='1de4c3df-a83d-480a-899b-fb263d3d5dd9', ...)]
 
     Or simply a convenient way to attach any necessary metadata to a model
-    that can be accessed by CustomJS callbacks, etc.
+    that can be accessed by ``CustomJS`` callbacks, etc.
 
     .. note::
         No uniqueness guarantees or other conditions are enforced on any tags
@@ -270,14 +315,14 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
     js_event_callbacks = Dict(String, List(Instance("bokeh.models.callbacks.CustomJS")),
     help="""
-    A mapping of event names to lists of CustomJS callbacks.
+    A mapping of event names to lists of ``CustomJS`` callbacks.
 
     Typically, rather then modifying this property directly, callbacks should be
     added using the ``Model.js_on_event`` method:
 
     .. code:: python
 
-        callback = CustomJS(code="console.log('tap event occured')")
+        callback = CustomJS(code="console.log('tap event occurred')")
         plot.js_on_event('tap', callback)
     """)
 
@@ -288,7 +333,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
     """)
 
     js_property_callbacks = Dict(String, List(Instance("bokeh.models.callbacks.CustomJS")), help="""
-    A mapping of attribute names to lists of CustomJS callbacks, to be set up on
+    A mapping of attribute names to lists of ``CustomJS`` callbacks, to be set up on
     BokehJS side when the document is created.
 
     Typically, rather then modifying this property directly, callbacks should be
@@ -301,12 +346,15 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
     """)
 
+    # Properties --------------------------------------------------------------
 
     @property
     def document(self):
         ''' The |Document| this model is attached to (can be ``None``)
 
         '''
+        if self._temp_document is not None:
+            return self._temp_document
         return self._document
 
     @property
@@ -328,17 +376,19 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
             return {
                 'type'    : self.__view_model__,
                 'subtype' : self.__subtype__,
-                'id'      : self._id,
+                'id'      : self.id,
             }
         else:
             return {
                 'type' : self.__view_model__,
-                'id'   : self._id,
+                'id'   : self.id,
             }
 
+    # Public methods ----------------------------------------------------------
+
     def js_on_event(self, event, *callbacks):
 
-        if not isinstance(event, str) and issubclass(event, Event):
+        if not isinstance(event, string_types) and issubclass(event, Event):
             event = event.event_name
 
         if event not in self.js_event_callbacks:
@@ -351,7 +401,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
 
     def js_on_change(self, event, *callbacks):
-        ''' Attach a CustomJS callback to an arbitrary BokehJS model event.
+        ''' Attach a ``CustomJS`` callback to an arbitrary BokehJS model event.
 
         On the BokehJS side, change events for model properties have the
         form ``"change:property_name"``. As a convenience, if the event name
@@ -406,11 +456,17 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
         Args:
             attr (str) : an attribute name on this object
-            callback (callable) : a callback function to register
+            *callbacks (callable) : callback functions to register
 
         Returns:
             None
 
+        Example:
+
+        .. code-block:: python
+
+            widget.on_change('value', callback1, callback2, ..., callback_n)
+
         '''
         if attr not in self.properties():
             raise ValueError("attempted to add a callback on nonexistent %s.%s property" % (self.__class__.__name__, attr))
@@ -478,7 +534,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         need to separately have the full attributes of those
         other objects.
 
-        There's no corresponding from_json() because to
+        There's no corresponding ``from_json()`` because to
         deserialize an object is normally done in the context of a
         Document (since the Document can resolve references).
 
@@ -500,7 +556,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         will need to separately have the full attributes of those
         other objects.
 
-        There's no corresponding from_json_string() because to
+        There's no corresponding ``from_json_string()`` because to
         deserialize an object is normally done in the context of a
         Document (since the Document can resolve references).
 
@@ -513,7 +569,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
         '''
         json_like = self._to_json_like(include_defaults=include_defaults)
-        json_like['id'] = self._id
+        json_like['id'] = self.id
         # serialize_json "fixes" the JSON from _to_json_like by converting
         # all types into plain JSON types # (it converts Model into refs,
         # for example).
@@ -527,7 +583,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         # The explicit assumption here is that hinted events do not need to
         # go through all the same invalidation steps. Currently this is the
         # case for ColumnsStreamedEvent and ColumnsPatchedEvent. However,
-        # this may need to be further refined in the future, if the a
+        # this may need to be further refined in the future, if the
         # assumption does not hold for future hinted events (e.g. the hint
         # could specify explicitly whether to do normal invalidation or not)
         if hint is None:
@@ -555,6 +611,13 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         self._document = doc
         self._update_event_callbacks()
 
+    @staticmethod
+    def _clear_extensions():
+        MetaModel.model_class_reverse_map = {
+            k:v for k,v in MetaModel.model_class_reverse_map.items()
+            if getattr(v, "__implementation__", None) is None
+        }
+
     def _detach_document(self):
         ''' Detach a model from a Bokeh |Document|.
 
@@ -575,7 +638,7 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
         That's what "json like" means.
 
         This method should be considered "private" or "protected",
-        for use internal to Bokeh; use to_json() instead because
+        for use internal to Bokeh; use ``to_json()`` instead because
         it gives you only plain JSON-compatible types.
 
         Args:
@@ -649,32 +712,9 @@ class Model(with_metaclass(MetaModel, HasProps, PropertyCallbackManager, EventCa
 
         return html
 
-    def _repr_pretty(self, p, cycle):
-        '''
-
-        '''
-        name = "%s.%s" % (self.__class__.__module__, self.__class__.__name__)
-        _id = getattr(self, "_id", None)
-
-        if cycle:
-            p.text(name)
-            p.text('(id=')
-            p.pretty(_id)
-            p.text(', ...)')
-        else:
-            with p.group(4, '%s(' % name, ')'):
-                props = self.properties_with_values().items()
-                sorted_props = sorted(props, key=itemgetter(0))
-                all_props = [('id', _id)] + sorted_props
-                for i, (prop, value) in enumerate(all_props):
-                    if i == 0:
-                        p.breakable('')
-                    else:
-                        p.text(',')
-                        p.breakable()
-                    p.text(prop)
-                    p.text('=')
-                    p.pretty(value)
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
 
 def _visit_immediate_value_references(value, visitor):
     ''' Visit all references to another Model without recursing into any
@@ -692,7 +732,6 @@ def _visit_immediate_value_references(value, visitor):
 
 _common_types = {int, float, str}
 
-
 def _visit_value_and_its_immediate_references(obj, visitor):
     ''' Recurse down Models, HasProps, and Python containers
 
@@ -717,3 +756,8 @@ def _visit_value_and_its_immediate_references(obj, visitor):
         else:
             # this isn't a Model, so recurse into it
             _visit_immediate_value_references(obj, visitor)
+
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
