@@ -1,17 +1,55 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Models for displaying maps in Bokeh plots.
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from ..core.enums import MapType
 from ..core.has_props import abstract
 from ..core.properties import Bool, Enum, Float, Instance, Int, JSON, Override, String
 from ..core.validation import error, warning
-from ..core.validation.warnings import MISSING_RENDERERS, NO_DATA_RENDERERS
+from ..core.validation.warnings import MISSING_RENDERERS
 from ..core.validation.errors import INCOMPATIBLE_MAP_RANGE_TYPE, REQUIRED_RANGE, MISSING_GOOGLE_API_KEY
 from ..model import Model
+from ..models.ranges import Range1d
 from .plots import Plot
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'GMapOptions',
+    'GMapPlot',
+    'MapOptions',
+    'MapPlot',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 @abstract
 class MapOptions(Model):
     ''' Abstract base class for map options' models.
@@ -52,12 +90,12 @@ class MapPlot(Plot):
             return "%s.y_range" % str(self)
 
 class GMapOptions(MapOptions):
-    ''' Options for GMapPlot objects.
+    ''' Options for ``GMapPlot`` objects.
 
     '''
 
     map_type = Enum(MapType, default="roadmap", help="""
-    The `map type`_ to use for the GMapPlot.
+    The `map type`_ to use for the ``GMapPlot``.
 
     .. _map type: https://developers.google.com/maps/documentation/javascript/reference#MapTypeId
 
@@ -68,7 +106,7 @@ class GMapOptions(MapOptions):
     """)
 
     styles = JSON(help="""
-    A JSON array of `map styles`_ to use for the GMapPlot. Many example styles can
+    A JSON array of `map styles`_ to use for the ``GMapPlot``. Many example styles can
     `be found here`_.
 
     .. _map styles: https://developers.google.com/maps/documentation/javascript/reference#MapTypeStyle
@@ -76,6 +114,16 @@ class GMapOptions(MapOptions):
 
     """)
 
+    tilt = Int(default=45, help="""
+    `Tilt`_ angle of the map. The only allowed values are 0 and 45.
+    Only has an effect on 'satellite' and 'hybrid' map types.
+    A value of 0 causes the map to always use a 0 degree overhead view.
+    A value of 45 causes the tilt angle to switch to 45 imagery if available.
+
+    .. _Tilt: https://developers.google.com/maps/documentation/javascript/reference/3/map#MapOptions.tilt
+
+    """)
+
 class GMapPlot(MapPlot):
     ''' A Bokeh Plot with a `Google Map`_ displayed underneath.
 
@@ -98,10 +146,6 @@ class GMapPlot(MapPlot):
     def _check_missing_renderers(self):
         pass
 
-    @warning(NO_DATA_RENDERERS)
-    def _check_no_data_renderers(self):
-        pass
-
     @error(MISSING_GOOGLE_API_KEY)
     def _check_missing_google_api_key(self):
         if self.api_key is None:
@@ -117,3 +161,19 @@ class GMapPlot(MapPlot):
     Google Maps API requires an API key. See https://developers.google.com/maps/documentation/javascript/get-api-key
     for more information on how to obtain your own.
     """)
+
+    x_range = Override(default=lambda: Range1d())
+
+    y_range = Override(default=lambda: Range1d())
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
