@@ -1,8 +1,30 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Various kinds of data table (data grid) widgets.
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from ...core.enums import DateFormat, FontStyle, NumeralLanguage, TextAlign, RoundingFunction
 from ...core.has_props import abstract
 from ...core.properties import Bool, Color, Either, Enum, Float, Instance, Int, List, Override, String
@@ -12,6 +34,36 @@ from ..sources import DataSource, CDSView
 
 from .widget import Widget
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'BooleanFormatter',
+    'CellFormatter',
+    'CellEditor',
+    'CheckboxEditor',
+    'DataTable',
+    'DateEditor',
+    'DateFormatter',
+    'HTMLTemplateFormatter',
+    'IntEditor',
+    'NumberEditor',
+    'NumberFormatter',
+    'PercentEditor',
+    'SelectEditor',
+    'StringEditor',
+    'StringFormatter',
+    'TableColumn',
+    'TextEditor',
+    'TimeEditor',
+    'TableWidget',
+)
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 @abstract
 class CellFormatter(Model):
     ''' Abstract base class for data table's cell formatters.
@@ -24,6 +76,10 @@ class CellEditor(Model):
 
     '''
 
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 class StringFormatter(CellFormatter):
     ''' Basic string cell formatter.
 
@@ -356,6 +412,8 @@ class HTMLTemplateFormatter(CellFormatter):
     The formatter has access other items in the row via the `dataContext` object passed to the formatter.
     So, for example, if another column in the datasource was named `url`, the template could access it as:
 
+    .. code-block:: jinja
+
         <a href="<%= url %>"><%= value %></a>
 
     To use a different set of template delimiters, pass the appropriate values for `evaluate`, `interpolate',
@@ -363,11 +421,17 @@ class HTMLTemplateFormatter(CellFormatter):
 
     Example: Simple HTML template to format the column value as code.
 
+    .. code-block:: python
+
         HTMLTemplateFormatter(template='<code><%= value %></code>')
 
     Example: Use values from other columns (`manufacturer` and `model`) to build a hyperlink.
 
-        HTMLTemplateFormatter(template='<a href="https:/www.google.com/search?q=<%= manufacturer %>+<%= model %>" target="_blank"><%= value %></a>')
+    .. code-block:: python
+
+        HTMLTemplateFormatter(template=
+            '<a href="https:/www.google.com/search?q=<%= manufacturer %>+<%= model %>" target="_blank"><%= value %></a>'
+        )
 
     '''
     template = String('<%= value %>', help="""
@@ -519,7 +583,7 @@ class DataTable(TableWidget):
     """)
 
     reorderable = Bool(True, help="""
-    Allows the reordering of a tables's columns. To reorder a column,
+    Allows the reordering of a table's columns. To reorder a column,
     click and drag a table's header to the desired location in the table.
     The columns on either side will remain in their previous order.
     """)
@@ -537,8 +601,24 @@ class DataTable(TableWidget):
     enabled) or using Shift + click on rows.
     """)
 
-    row_headers = Bool(True, help="""
-    Enable or disable row headers, i.e. the index column.
+    index_position = Int(0, help="""
+    Where among the list of columns to insert a column displaying the row
+    index. Negative indices are supported, and specify an index position
+    from the end of the list of columns (i.e. standard Python behaviour).
+
+    To prevent the index column from being added, set to None.
+
+    If the absolute value of index_position  is larger than the length of
+    the columns, then the index will appear at the beginning or end, depending
+    on the sign.
+    """)
+
+    index_header = String("#", help="""
+    The column header to display for the index column, if it is present.
+    """)
+
+    index_width = Int(40, help="""
+    The width of the index column, if present.
     """)
 
     scroll_to_selection = Bool(True, help="""
@@ -547,4 +627,20 @@ class DataTable(TableWidget):
     in the viewport.
     """)
 
+    header_row = Bool(True, help="""
+    Whether to show a header row with column names at the top of the table.
+    """)
+
     height = Override(default=400)
+
+    row_height = Int(25, help="""
+    The height of each row in pixels.
+    """)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
