@@ -1,16 +1,55 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import pytest ; pytest
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+import datetime
 from mock import mock
 
-import pandas as pd
-import pytest
+# External imports
+import numpy as np
 
+# Bokeh imports
 from bokeh.models import ColumnDataSource, CDSView, Marker
+from bokeh.models.axes import CategoricalAxis, LinearAxis, LogAxis, MercatorAxis, DatetimeAxis
 from bokeh.models.ranges import Range1d, DataRange1d, FactorRange
 from bokeh.models.scales import LinearScale, LogScale, CategoricalScale
 from bokeh.plotting import Figure
-from bokeh.plotting.helpers import _get_scale,_get_range, _stack, _graph, _glyph_function, _RENDERER_ARGS
 
+# Module under test
+from bokeh.plotting.helpers import _get_scale,_get_range, _stack, _graph, _glyph_function, _RENDERER_ARGS, _get_axis_class
 import bokeh.plotting.helpers as bph
 
+#-----------------------------------------------------------------------------
+# Setup
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
 class Test__get_legend_item_label(object):
 
     def test_legend_None(self):
@@ -79,7 +118,7 @@ def test__stack_broadcast_with_no_kwargs():
     kws = _stack(stackers, 'start', 'end')
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
-        assert set(['start', 'end']) == set(kw.keys())
+        assert set(['start', 'end', 'name']) == set(kw.keys())
         assert list(kw['start']['expr'].fields) == stackers[:i]
         assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
 
@@ -88,24 +127,51 @@ def test__stack_broadcast_with_scalar_kwargs():
     kws = _stack(stackers, 'start', 'end', foo=10, bar="baz")
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
-        assert set(['start', 'end', 'foo', 'bar']) == set(kw.keys())
+        assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
         assert list(kw['start']['expr'].fields) == stackers[:i]
         assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
         assert kw['foo'] == 10
         assert kw['bar'] == "baz"
+        assert kw['name'] == stackers[i]
 
 def test__stack_broadcast_with_list_kwargs():
     stackers = ['a', 'b', 'c', 'd']
     kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz")
     assert len(kws) == len(stackers)
     for i, kw in enumerate(kws):
-        assert set(['start', 'end', 'foo', 'bar']) == set(kw.keys())
+        assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
         assert list(kw['start']['expr'].fields) == stackers[:i]
         assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
         assert kw['foo'] == [10, 20, 30, 40][i]
         assert kw['bar'] == "baz"
+        assert kw['name'] == stackers[i]
 
-def test__graph_will_convert_dataframes_to_sources():
+def test__stack_broadcast_name_scalar_overrides():
+    stackers = ['a', 'b', 'c', 'd']
+    kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name="name")
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i]
+        assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
+        assert kw['foo'] == [10, 20, 30, 40][i]
+        assert kw['bar'] == "baz"
+        assert kw['name'] == "name"
+
+def test__stack_broadcast_name_list_overrides():
+    names = ["aa", "bb", "cc", "dd"]
+    stackers = ['a', 'b', 'c', 'd']
+    kws = _stack(stackers, 'start', 'end', foo=[10, 20, 30, 40], bar="baz", name=names)
+    assert len(kws) == len(stackers)
+    for i, kw in enumerate(kws):
+        assert set(['start', 'end', 'foo', 'bar', 'name']) == set(kw.keys())
+        assert list(kw['start']['expr'].fields) == stackers[:i]
+        assert list(kw['end']['expr'].fields) == stackers[:(i+1)]
+        assert kw['foo'] == [10, 20, 30, 40][i]
+        assert kw['bar'] == "baz"
+        assert kw['name'] == names[i]
+
+def test__graph_will_convert_dataframes_to_sources(pd):
     node_source = pd.DataFrame(data=dict(foo=[]))
     edge_source = pd.DataFrame(data=dict(start=[], end=[], bar=[]))
 
@@ -164,6 +230,59 @@ def test__graph_properly_handle_edge_property_mixins():
     assert r.hover_glyph.line_width == 23
     assert r.muted_glyph.line_width == 23
 
+_RANGES = [Range1d(), DataRange1d(), FactorRange()]
+
+class Test__get_axis_class(object):
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_None(self, range):
+        assert(_get_axis_class(None, range, 0)) == (None, {})
+        assert(_get_axis_class(None, range, 1)) == (None, {})
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_linear(self, range):
+        assert(_get_axis_class("linear", range, 0)) == (LinearAxis, {})
+        assert(_get_axis_class("linear", range, 1)) == (LinearAxis, {})
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_log(self, range):
+        assert(_get_axis_class("log", range, 0)) == (LogAxis, {})
+        assert(_get_axis_class("log", range, 1)) == (LogAxis, {})
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_datetime(self, range):
+        assert(_get_axis_class("datetime", range, 0)) == (DatetimeAxis, {})
+        assert(_get_axis_class("datetime", range, 1)) == (DatetimeAxis, {})
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_mercator(self, range):
+        assert(_get_axis_class("mercator", range, 0)) == (MercatorAxis, {'dimension': 'lon'})
+        assert(_get_axis_class("mercator", range, 1)) == (MercatorAxis, {'dimension': 'lat'})
+
+    def test_axis_type_auto(self):
+        assert(_get_axis_class("auto", FactorRange(), 0)) == (CategoricalAxis, {})
+        assert(_get_axis_class("auto", FactorRange(), 1)) == (CategoricalAxis, {})
+        assert(_get_axis_class("auto", DataRange1d(), 0)) == (LinearAxis, {})
+        assert(_get_axis_class("auto", DataRange1d(), 1)) == (LinearAxis, {})
+        assert(_get_axis_class("auto", Range1d(), 0)) == (LinearAxis, {})
+        assert(_get_axis_class("auto", Range1d(), 1)) == (LinearAxis, {})
+        assert(_get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 0)) == (DatetimeAxis, {})
+        assert(_get_axis_class("auto", Range1d(start=datetime.datetime(2018, 3, 21)), 1)) == (DatetimeAxis, {})
+
+
+    @pytest.mark.parametrize('range', _RANGES)
+    @pytest.mark.unit
+    def test_axis_type_error(self, range):
+        with pytest.raises(ValueError):
+            _get_axis_class("junk", range, 0)
+        with pytest.raises(ValueError):
+            _get_axis_class("junk", range, 1)
+
 def test__get_scale_numeric_range_linear_axis():
     s = _get_scale(Range1d(), "linear")
     assert isinstance(s, LinearScale)
@@ -193,19 +312,31 @@ def test__get_range_with_Range():
         assert r is rng
 
 def test__get_range_with_ndarray():
-    import numpy as np
     r = _get_range(np.array([10, 20]))
     assert isinstance(r, Range1d)
     assert r.start == 10
     assert r.end == 20
 
+def test__get_range_with_too_long_ndarray():
+    with pytest.raises(ValueError):
+        _get_range(np.array([10, 20, 30]))
+
 def test__get_range_with_ndarray_factors():
-    import numpy as np
     f = np.array(["Crosby", "Stills", "Nash", "Young"])
     r = _get_range(f)
     assert isinstance(r, FactorRange)
     assert r.factors == list(f)
 
+def test__get_range_with_series(pd):
+    r = _get_range(pd.Series([20, 30]))
+    assert isinstance(r, Range1d)
+    assert r.start == 20
+    assert r.end == 30
+
+def test__get_range_with_too_long_series(pd):
+    with pytest.raises(ValueError):
+        _get_range(pd.Series([20, 30, 40]))
+
 def test__get_range_with_string_seq():
     f = ["foo" ,"end", "baz"]
     for t in [list, tuple]:
@@ -225,7 +356,7 @@ def test__get_range_with_float_bounds():
     assert r.start == 1.2
     assert r.end == 10
 
-def test_get_range_with_pandas_group():
+def test_get_range_with_pandas_group(pd):
     from bokeh.sampledata.iris import flowers
     g = flowers.groupby('species')
     r = _get_range(g)
@@ -244,8 +375,8 @@ _renderer_args_values = {
     'visible': [None, False, True],
     'muted': [None, False, True]
 }
-@pytest.mark.parametrize('arg,values', [(arg, _renderer_args_values[arg])
-                                        for arg in _RENDERER_ARGS])
+@pytest.mark.parametrize('arg,values', [(arg, _renderer_args_values[arg]) for arg in _RENDERER_ARGS])
+@pytest.mark.unit
 def test__glyph_receives_renderer_arg(arg, values):
     for value in values:
         with mock.patch('bokeh.plotting.helpers.GlyphRenderer', autospec=True) as gr_mock:
@@ -253,3 +384,7 @@ def test__glyph_receives_renderer_arg(arg, values):
             fn(Figure(), x=0, y=0, **{arg: value})
             _, kwargs = gr_mock.call_args
             assert arg in kwargs and kwargs[arg] == value
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
