@@ -1,23 +1,52 @@
-from __future__ import absolute_import, print_function
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 
-from mock import patch
-import unittest
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+import pytest ; pytest
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 import logging
-import bokeh.document as document
-from bokeh.application import Application
-from bokeh.application.handlers import FunctionHandler
-from bokeh.client import pull_session, push_session, ClientSession
-from bokeh.document import Document
-from bokeh.model import Model
-from bokeh.models import Plot
-from bokeh.core.properties import Int, Instance, Dict, String, Any, DistanceSpec, AngleSpec
-from bokeh.document.events import ModelChangedEvent, TitleChangedEvent
+from mock import patch
+import os
+
+# External imports
 from tornado import gen
 from tornado.httpclient import HTTPError
 
+# Bokeh imports
+from bokeh.application import Application
+from bokeh.application.handlers import FunctionHandler
+from bokeh.client import pull_session, push_session, ClientSession
+import bokeh.document as document
+from bokeh.document import Document
+from bokeh.document.events import ModelChangedEvent, TitleChangedEvent
+from bokeh.core.properties import Int, Instance, Dict, String, Any, DistanceSpec, AngleSpec
+from bokeh.model import Model
+from bokeh.models import Plot
 from bokeh.server.tests.utils import ManagedServerLoop, url, ws_url, http_get, websocket_open
 
+# Module under test
+
+#-----------------------------------------------------------------------------
+# Setup
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 class AnotherModelInTestClientServer(Model):
     bar = Int(1)
 
@@ -35,7 +64,7 @@ class UnitsSpecModel(Model):
 
 logging.basicConfig(level=logging.DEBUG)
 
-class TestClientServer(unittest.TestCase):
+class TestClientServer(object):
 
     def test_minimal_connect_and_disconnect(self):
         application = Application()
@@ -88,13 +117,13 @@ class TestClientServer(unittest.TestCase):
             session.loop_until_closed(suppress_warning=True)
 
     def check_http_gets_fail(self, server):
-        with (self.assertRaises(HTTPError)):
+        with pytest.raises(HTTPError):
             http_get(server.io_loop, url(server))
-        with (self.assertRaises(HTTPError)):
+        with pytest.raises(HTTPError):
             http_get(server.io_loop, url(server) + "autoload.js?bokeh-autoload-element=foo")
 
     def check_connect_session_fails(self, server, origin):
-        with (self.assertRaises(HTTPError)):
+        with pytest.raises(HTTPError):
             websocket_open(server.io_loop,
                            ws_url(server)+"?bokeh-protocol-version=1.0&bokeh-session-id=foo",
                            origin=origin)
@@ -127,14 +156,32 @@ class TestClientServer(unittest.TestCase):
         with ManagedServerLoop(application, allow_websocket_origin=["example.com"]) as server:
             self.check_http_ok_socket_ok(server, origin="http://example.com:80")
 
+        # allow good origin from environment variable
+        with ManagedServerLoop(application) as server:
+            os.environ["BOKEH_ALLOW_WS_ORIGIN"] = "example.com"
+            self.check_http_ok_socket_ok(server, origin="http://example.com:80")
+            del os.environ["BOKEH_ALLOW_WS_ORIGIN"]
+
         # allow good origin with port
         with ManagedServerLoop(application, allow_websocket_origin=["example.com:8080"]) as server:
             self.check_http_ok_socket_ok(server, origin="http://example.com:8080")
 
+        # allow good origin with port from environment variable
+        with ManagedServerLoop(application) as server:
+            os.environ["BOKEH_ALLOW_WS_ORIGIN"] = "example.com:8080"
+            self.check_http_ok_socket_ok(server, origin="http://example.com:8080")
+            del os.environ["BOKEH_ALLOW_WS_ORIGIN"]
+
         # allow good origin header with an implicit 80
         with ManagedServerLoop(application, allow_websocket_origin=["example.com"]) as server:
             self.check_http_ok_socket_ok(server, origin="http://example.com")
 
+        # allow good origin header with an implicit 80
+        with ManagedServerLoop(application) as server:
+            os.environ["BOKEH_ALLOW_WS_ORIGIN"] = "example.com"
+            self.check_http_ok_socket_ok(server, origin="http://example.com")
+            del os.environ["BOKEH_ALLOW_WS_ORIGIN"]
+
         # block non-Host origins by default even if no extra origins specified
         with ManagedServerLoop(application) as server:
             self.check_http_ok_socket_blocked(server, origin="http://example.com:80")
@@ -147,10 +194,22 @@ class TestClientServer(unittest.TestCase):
         with ManagedServerLoop(application, allow_websocket_origin=["example.com"]) as server:
             self.check_http_ok_socket_blocked(server, origin="http://foobar.com:80")
 
+        # block bad origin from environment variable
+        with ManagedServerLoop(application) as server:
+            os.environ["BOKEH_ALLOW_WS_ORIGIN"] = "example.com"
+            self.check_http_ok_socket_blocked(server, origin="http://foobar.com:80")
+            del os.environ["BOKEH_ALLOW_WS_ORIGIN"]
+
         # block bad origin port
         with ManagedServerLoop(application, allow_websocket_origin=["example.com:8080"]) as server:
             self.check_http_ok_socket_blocked(server, origin="http://example.com:8081")
 
+        # block bad origin port from environment variable
+        with ManagedServerLoop(application) as server:
+            os.environ["BOKEH_ALLOW_WS_ORIGIN"] = "example.com:8080"
+            self.check_http_ok_socket_blocked(server, origin="http://example.com:8081")
+            del os.environ["BOKEH_ALLOW_WS_ORIGIN"]
+
     def test_push_document(self):
         application = Application()
         with ManagedServerLoop(application) as server:
@@ -248,13 +307,13 @@ class TestClientServer(unittest.TestCase):
             server._tornado._keep_alive() # send ping
             session.force_roundtrip()
 
-            self.assertEqual(expected_pong, connection._socket.latest_pong)
+            assert expected_pong == connection._socket.latest_pong
 
             # check that each ping increments by 1
             server._tornado._keep_alive()
             session.force_roundtrip()
 
-            self.assertEqual(expected_pong + 1, connection._socket.latest_pong)
+            assert (expected_pong + 1) == connection._socket.latest_pong
 
             session.close()
             session.loop_until_closed(suppress_warning=True)
@@ -403,11 +462,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 doc.remove_timeout_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_client_session_timeout_async_added_before_push(self):
         application = Application()
@@ -436,11 +495,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 doc.remove_timeout_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_server_session_timeout_async(self):
         application = Application()
@@ -472,11 +531,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 server_session.document.remove_timeout_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_client_session_next_tick_async(self):
         application = Application()
@@ -505,11 +564,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 doc.remove_next_tick_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_client_session_next_tick_async_added_before_push(self):
         application = Application()
@@ -538,11 +597,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 doc.remove_next_tick_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_server_session_next_tick_async(self):
         application = Application()
@@ -574,11 +633,11 @@ class TestClientServer(unittest.TestCase):
 
             client_session.loop_until_closed(suppress_warning=True)
 
-            with (self.assertRaises(ValueError)) as manager:
+            with pytest.raises(ValueError) as exc:
                 server_session.document.remove_next_tick_callback(cb_id)
-            self.assertTrue('already removed' in repr(manager.exception))
+            assert 'already removed' in repr(exc.value)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_client_session_periodic_async(self):
         application = Application()
@@ -609,7 +668,7 @@ class TestClientServer(unittest.TestCase):
 
             doc.remove_periodic_callback(cb_id)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_client_session_periodic_async_added_before_push(self):
         application = Application()
@@ -640,7 +699,7 @@ class TestClientServer(unittest.TestCase):
 
             doc.remove_periodic_callback(cb_id)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_server_session_periodic_async(self):
         application = Application()
@@ -674,7 +733,7 @@ class TestClientServer(unittest.TestCase):
 
             server_session.document.remove_periodic_callback(cb_id)
 
-            self.assertDictEqual(dict(a=0, b=1, c=2, d=3, e=4), result.values)
+            assert dict(a=0, b=1, c=2, d=3, e=4) == result.values
 
     def test_lots_of_concurrent_messages(self):
         application = Application()
@@ -765,8 +824,6 @@ class TestClientServer(unittest.TestCase):
             assert result['server_connection_count'] == 1
             assert result['server_close_code'] is None
 
-# This isn't in the unittest.TestCase because per-test fixtures
-# don't work there (see note at bottom of https://pytest.org/latest/unittest.html#unittest-testcase)
 def test_client_changes_do_not_boomerang(monkeypatch):
     application = Application()
     with ManagedServerLoop(application) as server:
@@ -813,8 +870,6 @@ def test_client_changes_do_not_boomerang(monkeypatch):
         assert not client_session.connected
         server.unlisten() # clean up so next test can run
 
-# This isn't in the unittest.TestCase because per-test fixtures
-# don't work there (see note at bottom of https://pytest.org/latest/unittest.html#unittest-testcase)
 def test_server_changes_do_not_boomerang(monkeypatch):
     application = Application()
     with ManagedServerLoop(application) as server:
@@ -930,3 +985,15 @@ def test_session_show_adds_obj_to_curdoc_if_necessary(m):
     assert session.document.roots == []
     session.show(p)
     assert session.document.roots == [p]
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
