@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide a collection of general utilities useful for implementing Bokeh
 functionality.
 
@@ -78,14 +84,6 @@ functionality.
 .. automodule:: bokeh.util.paths
   :members:
 
-.. _bokeh.util.plot_utils:
-
-``bokeh.util.plot_utils``
--------------------------
-
-.. automodule:: bokeh.util.plot_utils
-  :members:
-
 .. _bokeh.util.serialization:
 
 ``bokeh.util.serialization``
@@ -110,14 +108,6 @@ functionality.
 .. automodule:: bokeh.util.string
   :members:
 
-.. _bokeh.util.testing:
-
-``bokeh.util.testing``
-----------------------
-
-.. automodule:: bokeh.util.testing
-  :members:
-
 .. _bokeh.util.tornado:
 
 ``bokeh.util.tornado``
@@ -126,6 +116,14 @@ functionality.
 .. automodule:: bokeh.util.tornado
   :members:
 
+  .. _bokeh.util.terminal:
+
+``bokeh.util.terminal``
+-----------------------
+
+.. automodule:: bokeh.util.terminal
+  :members:
+
 .. _bokeh.util.version:
 
 ``bokeh.util.version``
