@@ -1,20 +1,49 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 """ Internal utils related to Tornado
 
 """
-from __future__ import absolute_import, print_function
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 import threading
 from collections import defaultdict
 
-log = logging.getLogger(__name__)
-
 from traceback import format_exception
 
+# External imports
+import six
 from tornado import gen
-from ..util.serialization import make_id
-from ..util import deprecation
 
+# Bokeh imports
+from ..util.serialization import make_id
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'yield_for_all_futures',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
 @gen.coroutine
 def yield_for_all_futures(result):
@@ -40,6 +69,13 @@ def yield_for_all_futures(result):
 
     raise gen.Return(result)
 
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
 
 class _AsyncPeriodic(object):
     """Like ioloop.PeriodicCallback except the 'func' can be async and
@@ -91,16 +127,20 @@ class _AsyncPeriodic(object):
         def on_done(future):
             if not self._stopped:
                 self._loop.add_future(invoke(), on_done)
-            if future.exception() is not None:
+            ex = future.exception()
+            if ex is not None:
                 log.error("Error thrown from periodic callback:")
-                log.error("".join(format_exception(*future.exc_info())))
+                if six.PY2:
+                    lines = format_exception(*future.exc_info())
+                else:
+                    lines = format_exception(ex.__class__, ex, ex.__traceback__)
+                log.error("".join(lines))
 
         self._loop.add_future(self.sleep(), on_done)
 
     def stop(self):
         self._stopped = True
 
-
 class _CallbackGroup(object):
     """ A collection of callbacks added to a Tornado IOLoop that we may
     want to remove as a group. """
@@ -140,9 +180,6 @@ class _CallbackGroup(object):
         else:
             raise RuntimeError('Unhandled removers', removers)
 
-    def _assign_deprecated_remover(self, callback, callback_id, removers):
-        self._get_removers_ids_by_callable(removers)[callback].add(callback_id)
-
     def _assign_remover(self, callback, callback_id, removers, remover):
         with self._removers_lock:
             if callback_id is None:
@@ -150,36 +187,19 @@ class _CallbackGroup(object):
             elif callback_id in removers:
                 raise ValueError("A callback of the same type has already been added with this ID")
             removers[callback_id] = remover
-            self._assign_deprecated_remover(callback, callback_id, removers)
             return callback_id
 
-    def _create_deprecated_remover(self, callback, removers):
-        deprecation.deprecated((0, 12, 15),
-                               'The ability to remove a callback function using its value',
-                               'a value returned from the function that adds a callback')
-        callback_ids = self._get_removers_ids_by_callable(removers).pop(callback)
-        to_run = [removers.pop(i) for i in callback_ids]
-
-        def remover():
-            for f in to_run:
-                f()
-
-        return remover
-
     def _execute_remover(self, callback_id, removers):
         try:
             with self._removers_lock:
-                if callable(callback_id):
-                    remover = self._create_deprecated_remover(callback_id, removers)
-                else:
-                    remover = removers.pop(callback_id)
-                    for cb, cb_ids in list(self._get_removers_ids_by_callable(removers).items()):
-                        try:
-                            cb_ids.remove(callback_id)
-                            if not cb_ids:
-                                del self._get_removers_ids_by_callable(removers)[cb]
-                        except KeyError:
-                            pass
+                remover = removers.pop(callback_id)
+                for cb, cb_ids in list(self._get_removers_ids_by_callable(removers).items()):
+                    try:
+                        cb_ids.remove(callback_id)
+                        if not cb_ids:
+                            del self._get_removers_ids_by_callable(removers)[cb]
+                    except KeyError:
+                        pass
         except KeyError:
             raise ValueError("Removing a callback twice (or after it's already been run)")
         remover()
@@ -244,3 +264,7 @@ class _CallbackGroup(object):
     def remove_periodic_callback(self, callback_id):
         """ Removes a callback added with add_periodic_callback."""
         self._execute_remover(callback_id, self._periodic_callback_removers)
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
